#!/usr/bin/env python
'''This script will parse the yum.log file generated by yum. And it will
   generate a nice pretty email from the information there depending on what you
   ask for. By default it does nothing.  If you do not set an action to capture,
   this script will do nothing.  If there is nothing to report for any enabled
   actions, this script silently exits, sending no email.  This script does not
   keep track of previous runs.  So if you desire, you can run it several times
   with different options for different groups to ensure you capture what each
   one needs without bothering the others.  Arguments can be specified in any
   order and some several times.  This program can also read from a
   configuration file. However, commandline arguments will override them.
'''

import ConfigParser
import datetime
import pprint
import re
import smtplib
import socket
import sys
from email.mime.text import MIMEText

try:
    from Cheetah.Template import Template
except ImportError:
    print >> sys.stderr, "Can't load the Cheetah template engine"
    print >> sys.stderr, " You will need to install that"
    print >> sys.stderr, " On SL6 simply run 'yum install python-cheetah'"
    print >> sys.stderr, " On SL5 you can find it in EPEL"
    print >> sys.stderr, ''
    print >> sys.stderr, " or at http://www.cheetahtemplate.org/"

try:
    import optparse
    # don't fix epilog white space, useful for manual formating control
    optparse.OptionParser.format_epilog = lambda self, formatter: self.epilog
except:
    print >> sys.stderr, "Can't load optparse"
    print >> sys.stderr, "I know it is Deprecated since 2.7"
    print >> sys.stderr, "I need to run on 2.4 and 26 which don't have argparse"

# This is a dictionary of defaults, it sits here at the top so it
# can be easily updated and is quick to locate
DEFAULTS = {
            'config':        '/dev/null',
            'showdefaults':  False,
            'defaultconfig': False,

            'installed':     False,
            'updated':       False,
            'erased':        False,
            'changed':       False,

            'package':       False,
            'notpackage':    False,
            'packagere':     False,
            'notpackagere':  False,

            'yumlog':        '/var/log/yum.log',
            'date':          datetime.datetime.now().strftime("%m/%d"),
            'smtphost':      '127.0.0.1',
            'cc':            False,
            'bcc':           False,
            'printout':      False,

            'subject':    '''#if ($GROUP != '')
Attention $GROUP: Changes on $HOSTNAME ($IPADDRESS), Please Review#slurp
#else
Changes on $HOSTNAME ($IPADDRESS), Please Review#slurp
#end if''',

            'template':   '''#if ($GROUP != '')
Hello $GROUP,
#else
Hello,
#end if

This is an automated email to inform you of changes to
$HOSTNAME ($IPADDRESS) on $TODAY.

#if ($INSTALLED != '')
The following packages were installed:
 #for $pkg in $INSTALLED
    $pkg
 #end for

#end if
#if ($ERASED != '')
The following packages were removed:
 #for $pkg in $ERASED
    $pkg
 #end for

#end if
#if ($UPDATED != '')
The following packages were updated:
 #for $pkg in $UPDATED
    $pkg
 #end for

#end if
The listed packages have been marked as important to your use
of this system.  Therefore, we are taking this oppertunity to
notify you of this change.
Please perform any validation testing your applications require
in this modified environment on $HOSTNAME ($IPADDRESS).

#if ($HELPDESK != '')
If there are any issues please inform
$HELPDESK
with all relevant information and a copy of this email.
#end if
Problems will need to be clearly described and include replication
steps so that we can assess the best way of correcting them.

If we do not hear of any problems within 7 days, by $WEEK1,
we will assume everything is working as expected and proceed as normal.

Thank you,
Your Admins#slurp
#if ($DEPARTMENT != '')
in $DEPARTMENT#slurp
#end if
.


You are recieving this message because you asked to be notified when
#if ($PACKAGELIST != '') or ($PACKAGERE != '')
any packages matching the following were changed:
$HOSTNAME ($IPADDRESS):
 #if ($PACKAGELIST != '')
  #for $pkg in $PACKAGELIST
    $pkg*
  #end for
 #end if
 #if ($PACKAGERE != '')
  #for $pkg in $PACKAGERE
    $pkg
  #end for
 #end if
#else
any packages were changed on
$HOSTNAME ($IPADDRESS).
#end if

Please let us know immediately if someone else should be getting this notice.
We can add as many people to the notification list as you think are necessary
to ensure everyone is informed and that testing is thorough and prompt.

Our goal is to help ensure the smooth operation of this system rather than
to send out meaningless notifications.  This validation testing is important
to everyone for keeping things working correctly.  If you are getting too
many notifications on packages you don't care about let us know.  The list
of important packages can be trimmed to include only package names that are
important to you.  However, you will need to tell us which package names
those are so we can make the necessary changes.  We believe the current list
to be an accurate representation of your notification needs.

''',
            'define':     [
                           'HELPDESK,'
                          ,'DEPARTMENT,'
                          ,'GROUP,'
                          ],
            'debug':      False,
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def build_option_list(default_values, debug_mode=False):
    '''This function builds the list of options that this program will accept.
    It returns an optparse object for use elsewhere.
    '''

    templatehelp = '''
  A final note:

Cheetah values to note for making a template:

 SUBJECT        The subject line after Cheetah has parsed it
 TO             Email addresses entered with --to=
 FROM           Email address entered with --from=
 CC             Email addresses entered with --cc= or '' if none
 TODAY          The date specified
 WEEK1          One week from TODAY
 WEEK2          Two weeks from TODAY
 WEEK3          Three weeks from TODAY
 WEEK4          Four weeks from TODAY
 WEEK5          Five weeks from TODAY
 WEEK6          Six weeks from TODAY
 WEEK7          Seven weeks from TODAY
 WEEK8          Eight weeks from TODAY
 DAYS30         30 days from TODAY
 DAYS60         60 days from TODAY
 HOSTNAME       The hostname of the system
 IPADDRESS      The IP address(es) of the system, comma seperated
 PACKAGELIST    A list of whatever you define with --package=
                 or "" if no packages are specified
 NOTPACKAGELIST A list of whatever you define with --notpackage=
                 or "" if no packages are specified
 PACKAGERE      A list of whatever you define with --packagere=
                 or "" if not specified
 NOTPACKAGERE   A list of whatever you define with --packagere=
                 or "" if not specified

 INSTALLED      A list of installed packages or "" if none
 ERASED         A list of erased packages or "" if none
 UPDATED        A list of updated packages or "" if none

If INSTALLED, ERASED, and UPDATED are all "" then no email will be sent out.
There are several checks in place to prevent that from happening.

The default template will accommodate:
 HELPDESK   - as some sort of contact information
               for getting first tier support
 DEPARTMENT - as your department name
 GROUP      - as the name of the group you are notifying
              but if they are not defined, they will skipped over
              cleanly so you don't notice.

You can use --define to override the defaults provided if, for example, you
think that HOSTNAME should contain some other value than the one it does.

  --define=HOSTNAME,myhost.example.com

Will change it for you This can come in handy for multihomed systems, or
boxes which have an alias that your users know that isn't the system's name.

Or you can set TO to a proper name, like:

  --define='TO,Web Development Department <webdev@example.com>'

While --to=webdev@example.com as an address is still required here.  This will
still email webdev@example.com, but give you the actual words in the template
so that people can be addressed by name rather than email address if you wish.

However, you cannot change INSTALLED ERASED UPDATED!

Don't change PACKAGELIST NOTPACKAGELIST PACKAGERE NOTPACKAGERE that will just
confuse people, but it is an option if you really really need to be able to do
that.  It will not override the commandline arguments that they are made from,
it will just make debugging really frustrating later on in life.

'''

    parser = optparse.OptionParser(description=__doc__, epilog=templatehelp)
    parser.set_defaults(**default_values)

    parser.add_option('--config', metavar="<FILENAME>",
                      help='''Path to an ini style config file.  The template
                            does not honor blank lines when placed directly
                            inline.  If you set template = /path/to/file
                            blank lines work as expected.  This limitation is
                            unfortunate, but consider it noted here.
                            There is no default.''')
    parser.add_option('--showdefaults', action='store_true',
                      help="Will print out the default values of everything.")
    parser.add_option('--defaultconfig', action='store_true',
                      help='''Will print out an ini style config file
                            containing the default values for this program.
                            It can be then be loaded with the --config=
                            option.''')

    actions = optparse.OptionGroup(parser, 'Actions',
              '''At least one of these options is required for anything useful
                 to be captured by this script.  Setting --changed is probably
                 what you want as it is the most complete record of changes.
                 The configuration file can set these in a '[actions]' section
                 (removing the -- from the front.''')
    actions.add_option('--installed', action='store_true',
                       help=''''Will capture the names of installed packages on
                             date.  The default is to ignore this action.''')
    actions.add_option('--updated', action='store_true',
                       help='''Will capture the names of updated packages on
                             date.  The default is to ignore this action.''')
    actions.add_option('--erased', action='store_true',
                       help='''Will capture the names of erased/removed
                             packages on date.  The default is to ignore this
                             action.''')
    actions.add_option('--changed', action='store_true',
                       help='''An alias for setting --installed --erased
                             --updated.   This is disabled by default.''')

    parser.add_option_group(actions)

    behavior = optparse.OptionGroup(parser, 'Behavior',
               '''You can modify this program's behavior in a number of ways.
                  The configuration file can set these options in a
                  '[behavior]' section (minus the --).''')
    behavior.add_option('--to', action="append", metavar='<REQUIRED>',
                        help='''Set who should recieve the email output.  You
                                can specify one address per invocation, but you
                                can specify extra addresses with extra
                                declarations.  There is no default.  This option
                                is required.''')
    behavior.add_option('--from', metavar='<REQUIRED>', dest='emailfrom',
                        help='''Set what is the "from" address you want for the
                                email.  There is no default.  This option is
                                required.''')
    behavior.add_option('--package', metavar="<STRING>",
                        help='''What package are we looking for?  Can be
                                specified as many times as you want.  It is,
                                however, CasE sEnsItiVe and matches from the
                                beginning of the package name, so for example,
                                a value of "gcc" will match gcc and gcc-g++.
                                This may lead to a few false positives, sorry.
                                It defaults to any package.''')
    behavior.add_option('--notpackage', metavar="<STRING>",
                        help='''This is the opposite of --package= and is used
                                to filter out packages that were matched by that
                                option.  You do not need to use --package= for
                                this to take effect, the same matching rules
                                still apply, however.  If you set --package=gcc
                                and --notpackage=gcc-g++ then notifications will
                                match gcc* except they wont match gcc-g++.  This
                                should help reduce your extra, accidental,
                                matches.  There is no default.''')
    behavior.add_option('--packagere', metavar="<REGEX>",
                        help='''A regular expression of packages to report on.
                                This is additive with --package= and can be set
                                multiple times if you wish.  There is no
                                default.''')
    behavior.add_option('--notpackagere', metavar="<REGEX>",
                        help='''This is the opposite of --packagere= and is used
                                to filter out packages that were matched by that
                                option.  It is additive with the --notpackage= 
                                option. You do not need to use any other options
                                to use this one.  It can be set multiple times.
                                There is no default.''')
    behavior.add_option('--subject', metavar="<TEMPLATE>",
                        help='''What should the subject line of the email read?
                                It is parsed by Cheetah before being used.  If
                                unspecified there is a default.  You can view it
                                with --showdefaults.''')
    behavior.add_option('--template', metavar="<TEMPLATE> or <FILENAME>",
                        help='''The name of your template file using Cheetah. If
                              unspecified there is a default.  You can view it
                              with --showdefaults.  You can set this to a raw
                              template (but please don't) or to a file
                              containing your template (do that instead) and the
                              script will understand what to do.''')
    behavior.add_option('--yumlog', metavar="<FILENAME>",
                        help='''Set the location for the yum.log to something
                                other than %default''')
    behavior.add_option('--date', metavar="MM/DD",
                        help='''The date to review, it looks for exactly one
                                day.  You must specify in numbers MM/DD if
                                setting this.  The above example should get you
                                started.  It also accepts a special value of
                                "yesterday" for examining the values from
                                yesterday.  It defaults to today (%default)''')

    behavior.add_option('--smtphost', metavar="<HOST>",
                        help='''This host should accept an unauthenticated SMTP
                                session from this system.  The default is
                                %default''')
    behavior.add_option('--cc', metavar="<CC>",
                        help='''Add this address as a 'CC' recipient.  Can use
                                multiple times to add as many addresses as you
                                want.  There is no default.''')
    behavior.add_option('--bcc', metavar="<BCC>",
                        help='''Add this address as a 'BCC' recipient.  Can use
                                multiple times to add as many addresses as you
                                want.  There is no default.''')
    behavior.add_option('--printout', action='store_true',
                        help='''Print to standard out a breif report of what was
                                emailed and to whom.  By default this script
                                prints only errors.''')
    behavior.add_option('--debug', action='store_true',
                        help="Enable debug mode, expect lots of text")

    parser.add_option_group(behavior)
        

    tdefine = optparse.OptionGroup(parser, 'Cheetah Control',
              '''You can pass things into Cheetah a few ways. The configuration
                 file can set these in a '[define]' section, in there it is
                 simply KEY=VALUE.  Documentation for Cheetah is avalible at
                 http://www.cheetahtemplate.org/ for review.''')

    tdefine.add_option('--define', action="append", metavar="<NAME,VALUE>",
                       help='''Set NAME,VALUE Can be used to define any value
                               you like.  The attribute is split on the first
                               comma.  Everything after the first ',' is treated
                               as the value of the entry.''')
    parser.add_option_group(tdefine)

    if debug_mode:
        print "Here are the default values for the options:"
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(default_values)

    return parser

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def validate_options(options, optparser):
    '''This function makes sure that we got the arguments we need and that
    they are the right sort of thing.
 
    It does extra work for --date and --template
    '''

    # doing special things here need to set date up first!
    now = datetime.datetime.now()
    if options.date == 'yesterday':
        day = datetime.timedelta(days=1)
        options.date = now - day
    else:
        try:
            temp = options.date.split('/')
            options.date = now.replace(month=int(temp[0]), day=int(temp[1]))
        except:
            optparser.error('Argument to --date invalid format, see --help for usage')
            sys.exit(1)

    # doing special things here need to set template up correctly
    if options.template.startswith('/'):
        try:
            cfgfile = open(options.template, 'r')
            options.template = cfgfile.read()
            cfgfile.close()
        except:
            print "Can't open template file", options.template
            sys.exit(1)

    # Now would be a good time to check for these as we need them.
    # We need them very badly!
    if not options.to:
        optparser.error('Required argument --to= missing, try --help for more info')
        sys.exit(1)
    if not options.emailfrom:
        optparser.error('Required argument --from= missing, try --help for more info')
        sys.exit(1)

    # this is what that option does
    if options.changed:
        options.installed = True
        options.updated = True
        options.erased = True

    # no point in going further if this is wrong
    if not options.installed:
        if not options.updated:
            if not options.erased:
                optparser.error("Ask for --installed, --updated, or --erased?  Try --help for more info")
                sys.exit(1)

    return options

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def read_config_file(configfile, debugmode):
    '''This function tries to read in the requested config file and returns
    recognized options in a dict that can be sent back into the argument parser.
    '''

    config = ConfigParser.SafeConfigParser()

    try:
        config.read(configfile)
    except ConfigParser.Error, err:
        print "Something is wrong with your config file"
        print str(err)
        sys.exit(1)

    values = {}
    for section in config.sections():
        if section == 'define':
            values['define'] = []
            for entry in config.options(section):
                values['define'].append(entry+','+config.get(section, entry))

        elif section == 'behavior':
            for entry in config.options(section):
                setting = config.get(section, entry)

                if setting.lower() == 'false':
                    values[entry] = False
                elif setting.lower() == 'true':
                    values[entry] = True
                # these options can be multiple times, so make them a list
                # if the conf is already a list, then use that list construct
                # otherwise make it a list here for later overrides
                elif entry in [ 'package', 'notpackage', 'packagere',
                                'notpackagere', 'to', 'cc', 'bcc']:
                    if setting.startswith('['):
                        values[entry] = eval(setting)
                    else:
                        values[entry] = []
                        values[entry].append(setting)
                else:
                    values[entry] = setting

        elif section == 'actions':
            for entry in config.options(section):
                values[entry] = config.get(section, entry)
                if values[entry].lower() == 'false':
                    values[entry] = False
                elif values[entry].lower() == 'true':
                    values[entry] = True

        elif section == 'template':
            if config.has_option(section,'message'):
                values['template'] = str(config.get(section, 'message'))
            if config.has_option(section,'subject'):
                values['subject'] = str(config.get(section, 'subject'))

    if debugmode:
        print "Config file (" + configfile + ") contained:"
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(values)

    return values

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def print_default_config():
    '''This function simply assembles a default config file and prints it
    to standard out, then exits.  This should provide a simple base
    for people wishing to make a config for controling this program.
    '''

    defconf = {}
    defconf['template'] = {}
    defconf['behavior'] = {}
    defconf['actions'] = {}
    defconf['define'] = {}
    defconf['behavior']['to'] = ''
    defconf['behavior']['from'] = ''
    for key in DEFAULTS.keys():
        if key not in ['template', 'subject', 'date', 'config',
                       'showdefaults', 'defaultconfig', 'installed',
                       'erased', 'updated', 'changed', 'define']:
            option = key.lstrip('-')
            defconf['behavior'][option] = DEFAULTS[key]
        elif key in ['installed', 'erased', 'updated', 'changed']:
            option = key.lstrip('-')
            defconf['actions'][option] = DEFAULTS[key]
        elif key == 'template':
            # strip blank lines as they don't work in the config!
            defconf['template']['message'] = str(DEFAULTS[key].replace('\n\n',
                                                     '\n')).replace('\n\n','\n')
        elif key == 'subject':
            defconf['template']['subject'] = DEFAULTS[key]
    defconf['define']['HELPDESK'] = '""'
    defconf['define']['GROUP'] = '""'
    defconf['define']['DEPARTMENT'] = '""'

    config = ConfigParser.ConfigParser()
    for section in defconf.keys():
        config.add_section(section)
        for entry in defconf[section].keys():
            config.set(section, entry, defconf[section][entry])

    print '#'
    print '# Any lines not present will have the value shown here.'
    print '# for setting a list of values simply use'
    print '# the default python list constructor on multivalued items'
    print '# package = [ "httpd", "python" ]'
    print '# check the program usage for which entries are multivalued'
    print '#'
    print '# You should use template=/some/file rather than an inline'
    print '# template as blank lines are not preserved when parsed here'
    print '#'
    print '# For simplicity you can remove any items you are leaving'
    print '# with their default values, this way the config will be nice'
    print '# and short so you can see what you do care about quickly'
    print '#'
    print ''
    config.write(sys.stdout)
    sys.exit(0)


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def read_yum_log(yumlogfile, date):
    '''This function opens the yum.log file and finds lines
    matching --date=.  Then it strips off the epoch as that
    has a tendency to confuse people and puts the rpm name
    in the right bucket

    You need to pass in full path to the logfile and the date
    you want checked - strftime("%b %d").
    '''

    try:
        log = open(yumlogfile, 'r')
    except:
        print "Can't read", yumlogfile
        sys.exit(1)

    if DEBUG:
        print 'Opened', yumlogfile
        print 'Looking for things matching', date

    packagelist = {}
    for line in log.readlines():
        line = line.rstrip()
        if line.startswith((date)):
            entry = line.split()
            # push the package name (no epoch) onto the stack
            # but only if it isn't already on the stack
            packageaction = entry[3]
            packagename = entry[4].lstrip('1234567890:')
            if not packagelist.has_key(packageaction):
                packagelist[packageaction] = []
            if packagename not in packagelist[packageaction]:
                packagelist[packageaction].append(packagename)
                if DEBUG:
                    print 'Adding', packagename, 'to', entry[3],'in packagelist'
    log.close()

    if DEBUG:
        print ''
        print 'Everything we found in', yumlogfile
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(packagelist)

    return packagelist

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def prune_package_list(options, packagelist):
    '''This function takes the packagelist dict and removes
    anything that you didn't ask for.
 
    The order of things is match, then remove so
    if you request gcc and remove gcc it will be removed.
 
    The end result is sorted because everyone likes sorted things
 
    You need to pass in options and packagelist
    '''

    if not options.installed:
        if packagelist.has_key('Installed:'):
            packagelist.pop('Installed:')

    if not options.updated:
        if packagelist.has_key('Updated:'):
            packagelist.pop('Updated:')

    if not options.erased:
        if packagelist.has_key('Erased:'):
            packagelist.pop('Erased:')

    if options.package or options.packagere:
        for key in packagelist.keys():
            temp = []
            if options.package:
                for package in options.package:
                    for changed in packagelist[key]:
                        if changed.startswith((package)):
                            # dedupe
                            if changed not in temp:
                                temp.append(changed)
                                if DEBUG:
                                    print '  ->', changed, "matches", package

            if options.packagere:
                for regex in options.packagere:
                    for package in packagelist[key]:
                        match_obj = re.match(regex, package)
                        if match_obj and package not in temp:
                            # dedupe
                            temp.append(package)
                            if DEBUG:
                                print '  ->', package, "matches", regex

            if len(temp) > 0:
                # since we will be making changes to temp[] we need
                # the value, not the reference
                packagelist[key] = temp[:]
                packagelist[key].sort()
            else:
                packagelist[key] = ''

    if options.notpackage or options.notpackagere:
        for key in packagelist.keys():
            temp = []
            if options.notpackage:
                for package in options.notpackage:
                    for changed in packagelist[key]:
                        if not changed.startswith((package)):
                            # dedupe
                            if changed not in temp:
                                temp.append(changed)
                                if DEBUG:
                                    print '  ->', changed, " not in", package
                # provide filtered list to regex parse if needed
                # since we will be making changes to temp[] we need
                # the value, not the reference
                packagelist[key] = temp[:]

            if options.notpackagere:
                for package in packagelist[key]:
                    match = False
                    for regex in options.notpackagere:
                        match_obj = re.match(regex, package)
                        if match_obj:
                            match = True
                            if DEBUG:
                                print '  ->', package, "matches", regex
                    if not match and package not in temp:
                        # dedupe
                        temp.append(package)

            if len(temp) > 0:
                # since we will be making changes to temp[] we need
                # the value, not the reference
                packagelist[key] = temp[:]
                packagelist[key].sort()
            else:
                packagelist[key] = ''

    for key in packagelist.keys():
        if packagelist[key] == '':
            packagelist.pop(key)

    if DEBUG:
        print "Packages we care about for operations that we track"
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(packagelist)

    return packagelist

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def send_notification(options, packagelist):
    '''This function takes everything we've learned so far and
    builds the report to send out
 
    Mostly this is just some final setup of the cheetah_vars
    and some calls to Cheetah, followed by an smtp session
 
    You need to pass in options and packagelist
    '''

    cheetah_vars = {}

    cheetah_vars['TO'] = ','.join(options.to)
    cheetah_vars['FROM'] = options.emailfrom
    if options.cc:
        cheetah_vars['CC'] = ','.join(options.cc)
    else:
        cheetah_vars['CC'] = ''

    # Provide the builtin defaults
    temp = socket.gethostbyaddr(socket.gethostname())
    cheetah_vars['HOSTNAME'] = temp[0]
    cheetah_vars['IPADDRESS'] = ','.join(temp[2])

    cheetah_vars['TODAY'] = options.date.strftime("%b %d, %Y")
    a_day = datetime.timedelta(days=1)
    weeks = {}
    for count in range(1, 9):
        weeks[count] = options.date + ((a_day * 7) * count)
        cheetah_vars['WEEK'+str(count)] = weeks[count].strftime("%b %d, %Y")

    cheetah_vars['DAYS30'] = options.date + (a_day * 30)
    cheetah_vars['DAYS30'] = cheetah_vars['DAYS30'].strftime("%b %d, %Y")

    cheetah_vars['DAYS60'] = options.date + (a_day * 60)
    cheetah_vars['DAYS60'] = cheetah_vars['DAYS60'].strftime("%b %d, %Y")

    if options.package:
        cheetah_vars['PACKAGELIST'] = options.package
    else:
        cheetah_vars['PACKAGELIST'] = ''

    if options.notpackage:
        cheetah_vars['NOTPACKAGELIST'] = options.notpackage
    else:
        cheetah_vars['NOTPACKAGELIST'] = ''

    if options.packagere:
        cheetah_vars['PACKAGERE'] = options.packagere
    else:
        cheetah_vars['PACKAGERE'] = ''

    if options.notpackagere:
        cheetah_vars['NOTPACKAGERE'] = options.notpackagere
    else:
        cheetah_vars['NOTPACKAGERE'] = ''

    # could be defined in the config file
    for defdefine in ['HELPDESK', 'GROUP', 'DEPARTMENT']:
        if not cheetah_vars.has_key(defdefine):
            cheetah_vars[defdefine] = ''

    # Do the necessary work to make this exportable to Cheetah
    for item in options.define:
        try:
            temp = item.split(',', 1)
            cheetah_vars[temp[0]] = temp[1]
        except:
            print 'One of your --define= arguments is not right, see usage'
            sys.exit(2)

    if packagelist.has_key('Installed:'):
        cheetah_vars['INSTALLED'] = packagelist['Installed:']
    else:
        cheetah_vars['INSTALLED'] = ''

    if packagelist.has_key('Erased:'):
        cheetah_vars['ERASED'] = packagelist['Erased:']
    else:
        cheetah_vars['ERASED'] = ''

    if packagelist.has_key('Updated:'):
        cheetah_vars['UPDATED'] = packagelist['Updated:']
    else:
        cheetah_vars['UPDATED'] = ''

    if DEBUG:
        print "Cheetah will have the following searchList"
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(cheetah_vars)

    subject = Template(options.subject, searchList=[cheetah_vars])

    # not too sure why we would want the subject in the message body
    # but it wasn't hard to make possible, so.....
    cheetah_vars['SUBJECT'] = str(subject)

    message = Template(options.template, searchList=[cheetah_vars])

    email = MIMEText(str(message))
    email['Subject'] = str(subject)
    email['From'] = options.emailfrom
    email['To'] = ','.join(options.to)
    if options.cc:
        email['CC'] = ','.join(options.cc)
    if options.bcc:
        email['BCC'] = ','.join(options.bcc)

    send = smtplib.SMTP(options.smtphost)
    send.sendmail(email['From'], email['To'], email.as_string())
    send.quit()

    if DEBUG:
        print 'Email sent'
        print email.as_string()

    # if we asked for a mini report, provide it now
    if options.printout:
        print_summary(cheetah_vars, packagelist, email)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
def print_summary(cheetah_vars, packagelist, email):
    ''' This function takes everything we've learned so far and
    provides a simple printout of what we have learned so far.
 
    Need to pass it our Cheetah Vars, packagelist, and what our email said.
    '''

    print 'Email sent:'
    print '-----------'
    print 'TO:', email['To']
    for cctype in ['CC','BCC']:
        if email.has_key(cctype):
            print cctype + ':', email[cctype]

    for key in packagelist.keys():
        print key
        for pkg in packagelist[key]:
            print ' ', pkg
    print ''
    for item in ['PACKAGELIST', 'PACKAGERE', 'NOTPACKAGELIST', 'NOTPACKAGERE']:
        if isinstance(cheetah_vars[item], list):
            print item +': ',
            for expression in cheetah_vars[item]:
                print expression,
            print ''

##############################################################################

DEBUG = False

def main():
    '''I dislike having much in global scope so, main()'''

    optparser = build_option_list(default_values = DEFAULTS,
                                 debug_mode = DEFAULTS['debug'])
    # have no 'args' that are position dependent, need return item [0]
    options = optparser.parse_args()[0]

    if options.config:
        configfile_values = read_config_file(options.config, options.debug)
        updated_defaults = DEFAULTS.copy()
        updated_defaults.update(configfile_values)
        optparser = build_option_list(default_values = updated_defaults,
                                      debug_mode = options.debug)
        # have no 'args' that are position dependent, need return item [0]
        options = optparser.parse_args()[0]

    # U.G.L.Y. you ain't got no alibi you're ugly, yeah yeah, you're ugly!
    #  ...but terribly convenient.
    #  if DEBUG: is just such a simple construct.
    global DEBUG
    DEBUG = options.debug

    if DEBUG:
        print "Options set:"
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(options)

    if options.defaultconfig:
        print_default_config()
        sys.exit(0)

    if options.showdefaults:
        _pp = pprint.PrettyPrinter(indent=4)
        _pp.pprint(DEFAULTS)
        sys.exit(0)

    options = validate_options(options, optparser)

    packages = read_yum_log(options.yumlog,
                            options.date.strftime("%b %d"))

    packagelist = prune_package_list(options, packages)

    if not len(packagelist.keys()) > 0:
        if DEBUG:
            print 'Nothing found to report'
        sys.exit(0)

    send_notification(options, packagelist)

    sys.exit(0)

if __name__ == "__main__": 
    main()
